-- Heal Command (HealComm)
-- By Moonlit - Myzrael (US) (A)
-- Based on HealOfficer by Tytus_ttv, which was itself based on SurgeonGeneral by Brimmstone 

---------------------------------------
-- TODO
---------------------------------------

--[[

- Ideas:
- Auto whisper assignment for each healer on report / boss target 
- Prompt to change assignment when boss targeted
-- maybe: ability to change preset assignments 

]]


---------------------------------------
-- INITS 
---------------------------------------
HealComm = LibStub("AceAddon-3.0"):NewAddon("HealComm", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0")
local LibDBIcon = LibStub("LibDBIcon-1.0")
local HealCommLDB = LibStub("LibDataBroker-1.1"):NewDataObject("HealComm", {
   type = "launcher",
   text = "HC",
   icon = "Interface\\Icons\\Spell_ChargePositive",
   OnTooltipShow = function(tooltip)
	   tooltip:AddLine("HealComm")
      tooltip:AddLine("Open HealComm")
	end,
   OnClick = function()
      HealComm:ToggleFrame()
   end,
})


local raidHealers = {}
local raidTanks = {}
local defaults = {
   profile = {
      minimap = {
         hide = false
      },
      broadcastToRaid = false, 
      whisperAssignments = false, 
      broadcastToCustomChannel = false,
      customChannelName = "",
      exceptionList = "",
      autoReplyAssignments = true, 
      highlightMissingHealers = true,
      broadcastHealerNamesFirst = false,
      assignments = {},
   },
}
function HealComm:OnInitialize()
   self.db = LibStub("AceDB-3.0"):New("HealCommDB", defaults)
   LibDBIcon:Register("HealComm", HealCommLDB, self.db.profile.minimap)
   self:RegisterChatCommand("healcomm", "ToggleFrame")
   self:RegisterChatCommand("heal", "ToggleFrame")
   self:RegisterChatCommand("hc", "ToggleFrame") 
   self:RegisterOptions()
   self:RegisterEvent("GROUP_ROSTER_UPDATE", "UpdateRoster")
   if self.db.profile.autoReplyAssignments then 
      self:RegisterEvent("CHAT_MSG_WHISPER", "ReplyWithAssignment")
   end 
end

function HealComm:OnEnable()
   self.debug = false
   self:CreateFrame()
   if not self.debug then 
      self.frame:Hide()
   end

   -- if in raid and we just load the interface, need to scan raid and update raid tables 
   if IsInRaid() then 
      self:UpdateRoster()
   end 
end

function HealComm:OnDisable()
end

function HealComm:ToggleFrame()
   if self.frame:IsVisible() then
      self.frame:Hide()
   else
      self.frame:Show()
      self.frame.broadcastToRaidCheckbox:SetChecked(self.db.profile.broadcastToRaid)
      self.frame.whisperAssignmentsCheckbox:SetChecked(self.db.profile.whisperAssignments)
      self.frame.broadcastToCustomChannelCheckbox:SetChecked(self.db.profile.broadcastToCustomChannel)
      self.frame.customChannelName:SetText(self.db.profile.customChannelName)
      self.frame.exceptionList:SetText(self.db.profile.exceptionList)
      self:UpdateRoster()
   end
end

---------------------------------------
-- COMMS 
---------------------------------------
function HealComm:Print(...)
   print("|cff33ff99HealComm: |cffffffff", ...)
end

function HealComm:DebugPrint(...)
   if self.debug then
         print("|cff33ff99HealComm Debug: |cffffffff", ...)
   end
end

-- Broadcast figures out what to broadcast and which channel to send it to 
function HealComm:Broadcast()
   local selectedEncounter = self:GetCurrentlySelectedRaidAndBoss()
   local btrim1, btrim2 = string.find(selectedEncounter, "%s%-%s")
   local bcastBoss = string.sub(selectedEncounter, btrim2+1)
   -- validation 
   if not selectedEncounter then
      self:Print("Raid and boss must be selected in order to broadcast assignments.") 
      return
   end 
   if not self.frame.broadcastToRaidCheckbox:GetChecked() and not self.frame.broadcastToCustomChannelCheckbox:GetChecked() and not self.frame.whisperAssignmentsCheckbox:GetChecked() then
      self:Print("Select the channels to broadcast the assignments to.")
      return 
   elseif self.frame.broadcastToRaidCheckbox:GetChecked() and not IsInRaid() then 
      self:Print("You must be in a raid to broadcast to the raid channel.")
      return
   elseif self.frame.broadcastToCustomChannelCheckbox:GetChecked() and self.frame.customChannelName:GetText() == "" then
      self:Print("Custom channel name is mandatory if broadcasting to a custom channel.")
      return 
   end 
   self:Print("Broadcasting assignments for", selectedEncounter)
   if self.frame.broadcastToRaidCheckbox:GetChecked() or self.frame.broadcastToCustomChannelCheckbox:GetChecked() then 
      -- begin broadcast 
      self:BroadcastMessage(" ")
      self:BroadcastMessage("== Healing Assignments: "..bcastBoss.." ==")
      self:BroadcastMessage(" ")
                  
      -- group healers by assignments and phases 
      local assignmentsMesssage = {phase1={},phase2={},phase3={}}
      for phase=1,3 do 
         -- tells us whether to broadcast this phase
         local numAssignmentsInPhase = 0  

         for row=1,15 do
            -- get assignment values from editboxes
            local assignment = self.frame.assignments["phase"..phase][row]:GetText()

            -- resolve %MT tokens into player names if they exist         
            assignment = string.gsub(assignment, "%%MT(%d+)", function(w) return self:ResolveTankName(w) end)
            -- append raid icon if assigned 
            assignment = self:AppendRaidIconToName(assignment)

            -- create storage table if assignment isnt blank and table doesnt exist
            if strlen(assignment) > 0 and not assignmentsMesssage["phase"..phase][assignment] then 
               assignmentsMesssage["phase"..phase][assignment] = {}
            end

            -- add healer to storage table if a name exists
            local healer = self.frame.healerNames[row]:GetText()
            if strlen(healer) > 0 and strlen(assignment) > 0 then 
               tinsert(assignmentsMesssage["phase"..phase][assignment], healer) 
               numAssignmentsInPhase = numAssignmentsInPhase + 1 
            end    
         end
         
         -- check if this phase has any assignments, otherwise skip 
         if numAssignmentsInPhase > 0 then
            if self.frame.Phase1Checkbox:GetChecked() or self.frame.Phase2Checkbox:GetChecked() then
               pInd2 = tonumber(phaseLabel2:GetText():sub(2,2))
               pInd3 = tonumber(phaseLabel3:GetText():sub(2,2))
               if phase == 1 then 
                  self:BroadcastMessage("---Phase 1---")
               elseif (phase == 2 and pInd2 > 1) or (phase == 3 and pInd2 == 1 and pInd3 == 2) then
                  self:BroadcastMessage("---Phase 2---")
               elseif phase == 3 and pInd3 > 2 then 
                  self:BroadcastMessage("---Phase 3---")
               end   
            end
            for k, v in pairs(assignmentsMesssage["phase"..phase]) do 
               if self.db.profile.broadcastHealerNamesFirst == true then
                  self:BroadcastMessage(table.concat(v,", ")..": "..k)
               else 
                  self:BroadcastMessage(k..": "..table.concat(v,", ")) 
               end

            end
         end 
      end
      local noteText = self.frame.notes:GetText()
      self:BroadcastMessage(" ")
      self:BroadcastMessage(noteText)
      self:BroadcastMessage(" ")
      if self.db.profile.autoReplyAssignments then
         self:BroadcastMessage("(Whisper me \"assign\" or \"repeat\" at any time.)")            
      end
      self:BroadcastMessage(" ")
      
   end 

   if self.frame.whisperAssignmentsCheckbox:GetChecked() then 
      self:WhisperAssignments()
   end 
end

-- attempts to look up the name of a tank given an %MT# token 
function HealComm:ResolveTankName(tankIndex)
   if not tankIndex then
      return nil 
   end 
   local tankIndex = tonumber(tankIndex) 
   if raidTanks[tankIndex] then
      return raidTanks[tankIndex]
   else 
      return "MT"..tankIndex
   end 
end

-- checks if the given player name has a raid target icon assigned to them and returns the name plus icon
function HealComm:AppendRaidIconToName(name)
   if UnitExists(name) then
      local icon = GetRaidTargetIndex(name)
      if icon then 
         name = name.." {rt"..icon.."}"
      end 
   end
   return name     
end
-- BroadcastMessage simply sends the given message to a channel
-- Used by channel broadcast function above 
function HealComm:BroadcastMessage(message)
   if self.db.profile.broadcastToRaid then 
      SendChatMessage(message, "RAID", nil, nil)
   end 

   if self.db.profile.broadcastToCustomChannel and strlen(self.db.profile.customChannelName) > 0 then 
      local channelNumber, _ = GetChannelName(self.db.profile.customChannelName)
      if channelNumber == 0 then return end 
      SendChatMessage(message, "CHANNEL", nil, channelNumber)
   end 
end 

-- WhisperAssignments finds the assignments for all healers in the raid and whispers it to them 
function HealComm:WhisperAssignments()
   if not self.db.profile.whisperAssignments then return end 

   for i=1,15 do 
      local name = self.frame.healerNames[i]:GetText()
      if strlen(name) > 0 and self:IsHealerInRaid(name) then 
         self:SendAssignmentWhisperToPlayer(i, name)
      end
   end 
end 

-- listens for whisper trigger and replies to the person with only their current assignment 
-- will only send to people in your raid group 
function HealComm:ReplyWithAssignment(event, message, player)
   if not self.db.profile.autoReplyAssignments then return end 

   -- get rid of server tag 
   local player = string.gsub(player, "-(.*)", "")

   --if message == "assignment" and UnitInRaid(player) then 
   message = string.lower(message)
   if message:sub(1, 6) == "assign" or message:sub(1, 6) == "repeat" then 
      self:DebugPrint("Received assignment request from", player)
      local selectedEncounter = self:GetCurrentlySelectedRaidAndBoss()
      if not selectedEncounter then 
         SendChatMessage("HealComm: No encounter currently selected", "WHISPER",nil, player)
         return 
      end 
      for i=1,15 do 
         if self.frame.healerNames[i]:GetText() == player then 
            self:SendAssignmentWhisperToPlayer(i, player)
            return 
         end 
      end
      -- if not found, reply to player telling them so 
      local noteText = self.frame.notes:GetText()
      SendChatMessage("HealComm: No specific assignments found for "..player..". Encounter: "..selectedEncounter..".  However, the following generic notes were broadcast:", "WHISPER",nil, player)
      SendChatMessage(noteText, "WHISPER",nil, player)
   end 
end

function HealComm:SendAssignmentWhisperToPlayer(healerId, healerName)
   local response = ""
   local p1assignment = self.frame.assignments["phase1"][healerId]:GetText()
   local p2assignment = self.frame.assignments["phase2"][healerId]:GetText()
   local p3assignment = self.frame.assignments["phase3"][healerId]:GetText()
   pInd2 = tonumber(phaseLabel2:GetText():sub(2,2))
   pInd3 = tonumber(phaseLabel3:GetText():sub(2,2))
      
   if strlen(p1assignment) > 0 then 
      if pInd3 > 1 then response = " Phase 1:" end
      response = response.." "..p1assignment
   end
   if strlen(p2assignment) > 0 then
      if pInd2 > 1 then
         response = response.." -- Phase 2:"
      else
         response = response..","
      end
      response = response.." "..p2assignment
   end
   if strlen(p3assignment) > 0 then
      if pInd3 > pInd2 then
         response = response.." -- Phase "..pInd3..":"
      else
         response = response..","
      end
      response = response.." "..p3assignment
   end
   if strlen(response) == 0 then 
      response = "No assignment"
   end 
   local selectedEncounter = self:GetCurrentlySelectedRaidAndBoss()
   local btrim1, btrim2 = string.find(selectedEncounter, "%s%-%s")
   local bcastBoss = string.sub(selectedEncounter, btrim2+1)
   
   SendChatMessage(healerName.."'s Healing Assignment for "..bcastBoss..":", "WHISPER",nil, healerName)
   SendChatMessage("== "..response.." ==", "WHISPER",nil, healerName)
end

---------------------------------------
-- DATA HANDLING 
---------------------------------------
function HealComm:SaveAssignments()
   local selectedEncounter = self:GetCurrentlySelectedRaidAndBoss()
   self:DebugPrint("Saving assignments for", selectedEncounter)

   if not selectedEncounter then
      self:Print("Raid and boss must be selected in order to save assignments.") 
      return
   end 
   if not self.db.profile.assignments[selectedEncounter] then
      self.db.profile.assignments[selectedEncounter] = {}
      self.db.profile.assignments[selectedEncounter].healers = {}
      self.db.profile.assignments[selectedEncounter].assignments = {}
      self.db.profile.assignments[selectedEncounter].notes = {}
      self.db.profile.assignments[selectedEncounter].phases = {}
   end 
   for i=1,15 do 
      -- save healer names
      self.db.profile.assignments[selectedEncounter].healers[i] = (string.lower(self.frame.healerNames[i]:GetText())):gsub("^%l", string.upper)
   
      -- save assignments by phase, ignore blank values (hence the if statements)
      for j=1,3 do 
            if not self.db.profile.assignments[selectedEncounter].assignments["phase"..j] then
              self.db.profile.assignments[selectedEncounter].assignments["phase"..j] = {} 
            end 
            self.db.profile.assignments[selectedEncounter].assignments["phase"..j][i] = self.frame.assignments["phase"..j][i]:GetText()
      end 
   end 
   local phaseCount = 0
   if self.frame.Phase1Checkbox:GetChecked() then 
      phaseCount = phaseCount + 1
   end 
   if self.frame.Phase2Checkbox:GetChecked() then
      phaseCount = phaseCount + 2
   end
   self.db.profile.assignments[selectedEncounter].phases = phaseCount  

   self.db.profile.assignments[selectedEncounter].notes = self.frame.notes:GetText()
   self:Print("Saved assignments for", selectedEncounter)
end
function HealComm:LoadAssignments()
   local selectedEncounter = self:GetCurrentlySelectedRaidAndBoss()
   self:DebugPrint("Loading assignments for", self:GetCurrentlySelectedRaidAndBoss())
      
   if not self.db.profile.assignments[selectedEncounter] then
      self:Print("Keeping current assignments. No assignments saved for", selectedEncounter)
      return
   end 
   for i=1,15 do 
      -- load healer names, include blanks to clear past encounter assignments
      local healerName = self.db.profile.assignments[selectedEncounter].healers[i]
      if healerName then 
         self.frame.healerNames[i]:SetText(healerName)
      else
         self.frame.healerNames[i]:SetText("")
      end 

      -- load assignments by phase 
      for j=1,3 do 
         if self.db.profile.assignments[selectedEncounter].assignments["phase"..j] then 
            local assignment = self.db.profile.assignments[selectedEncounter].assignments["phase"..j][i]
            if assignment then 
               self.frame.assignments["phase"..j][i]:SetText(assignment)
            end 
         end 
      end 
   end
   local phaseCount = self.db.profile.assignments[selectedEncounter].phases
   if phaseCount == 1 or phaseCount == 3 then 
      self.frame.Phase1Checkbox:SetChecked(true)
   else
      self.frame.Phase1Checkbox:SetChecked(false)
   end
   if phaseCount == 2 or phaseCount == 3 then
      self.frame.Phase2Checkbox:SetChecked(true)
   else 
      self.frame.Phase2Checkbox:SetChecked(false)
   end
   self.frame.notes:SetText(self.db.profile.assignments[selectedEncounter].notes)
   self:UpdatePhaseLabels()
   self:Print("Loaded assignments for", selectedEncounter)
end

function HealComm:ClearAssignments()
   for i=1, 15 do 
      self.frame.healerNames[i]:SetText("")

      for j=1,3 do
         self.frame.assignments["phase"..j][i]:SetText("")
      end 
   end
end

function HealComm:UpdatePhaseLabels()
   if self.frame.Phase1Checkbox:GetChecked() then
      phaseLabel2:SetText("P2: Assign 1")
   else
      phaseLabel2:SetText("P1: Assign 2")
   end
   if self.frame.Phase1Checkbox:GetChecked() and not self.frame.Phase2Checkbox:GetChecked() then
      phaseLabel3:SetText("P2: Assign 2")
   elseif self.frame.Phase1Checkbox:GetChecked() and self.frame.Phase2Checkbox:GetChecked() then
      phaseLabel3:SetText("P3: Assign 1")
   elseif not self.frame.Phase1Checkbox:GetChecked() and self.frame.Phase2Checkbox:GetChecked() then
      phaseLabel3:SetText("P2: Assign 1")
   elseif not self.frame.Phase1Checkbox:GetChecked() and not self.frame.Phase2Checkbox:GetChecked() then
      phaseLabel3:SetText("P1: Assign 3")
   end
   return
end

-- save channel reporting settings, persist this outside of the assignments data tables as it shouldn't change
function HealComm:SaveReportingSettings()
   self.db.profile.broadcastToRaid = self.frame.broadcastToRaidCheckbox:GetChecked()
   self.db.profile.broadcastToCustomChannel = self.frame.broadcastToCustomChannelCheckbox:GetChecked()
   self.db.profile.whisperAssignments = self.frame.whisperAssignmentsCheckbox:GetChecked()
   self.db.profile.customChannelName = self.frame.customChannelName:GetText()
   --self.db.profile.exceptionList = self.frame.exceptionList:GetText()
end

function HealComm:UpdateExceptions()
   self.db.profile.exceptionList = self.frame.exceptionList:GetText()
   self.frame.exceptionList:SetText(self.db.profile.exceptionList)
   self:UpdateRoster()
end   

function HealComm:UpdateRoster()
   raidHealers = {}
   raidTanks = {}
   if not IsInRaid() then
      self:DebugPrint("UpdateRoster not in raid, stopping")
      return 
   end 
   for i=1, GetNumGroupMembers() do 
      local name, _, _, _, _, class, _, _, _, role = GetRaidRosterInfo(i) -- class variable is pulling from "FileName" because "Class" is localized.
      if role == "MAINTANK" then
         tinsert(raidTanks, name) 
      elseif (class == "PRIEST" or class == "DRUID" or class == "SHAMAN" or class == "PALADIN" or class == "MONK") then
         local skip = 0
         local Exceptions = {}
         local exc = self.frame.exceptionList:GetText()
--         exc = string.gsub(" "..string.lower(exc), "%W%l", string.upper):sub(2)
--         self.frame.exceptionList:SetText(exc)
--         for substring in exc:gmatch("%S+") do
         for substring in exc:gmatch("[^, *]+") do
           table.insert(Exceptions, substring)
         end
         for k,v in pairs(Exceptions) do
            if string.lower(name) == string.lower(v) then
              skip = 1
            end
         end
         if skip ~= 1 then
            tinsert(raidHealers, {name = name, assigned = false})
         end
      end 
   end
   self:UpdatePresetAssignmentTankNames()
end

---------------------------------------
-- INTERFACE EVENTS 
---------------------------------------
function HealComm:HealerNameChanged(healerId)
   local name = self.frame.healerNames[healerId]:GetText()
   self.frame.healerNames[healerId]:SetText(string.lower(name):gsub("^%l", string.upper))
   local _, class, _ = UnitClass(name)

   -- update healer icon if we know the class 
   if class ~= nil then 
      local coords = CLASS_ICON_TCOORDS[class] -- get the coordinates of the class icon we want
      self.frame.healerIcons[healerId].texture:SetTexture("Interface\\GLUES\\CHARACTERCREATE\\UI-CHARACTERCREATE-CLASSES") -- this is the image containing all class icons
      self.frame.healerIcons[healerId].texture:SetTexCoord(unpack(coords)) -- cut out the region with our class icon according to coords
      self.frame.healerIcons[healerId].texture:SetAllPoints()
      self.frame.healerIcons[healerId]:Show()
   else 
      self.frame.healerIcons[healerId].texture:SetTexture(nil)
      self.frame.healerIcons[healerId]:Hide()
   end 

   -- set name to red if healer is not in current raid 
   -- if self.db.highlightMissingHealers == true and strlen(name) > 0 and #raidHealers > 0 and not self:IsHealerInRaid(name) then 
   if self.db.profile.highlightMissingHealers == true and strlen(name) > 0 and #raidHealers > 0 and not self:IsHealerInRaid(name) then 
      self.frame.healerNames[healerId]:SetTextColor(1, 0, 0)
   else 

      -- default to white text out of raid for ease of readibility 
      self.frame.healerNames[healerId]:SetTextColor(1, 1, 1)
   end 
end

function HealComm:IsHealerInRaid(name)
   for k, v in pairs(raidHealers) do
      if v.name == name then 
         return true 
      end
   end
   return false
end

function HealComm:AutoFillHealers()
   if #raidHealers == 0 then 
      self:Print("Not in a raid or no eligible healers.")
      return 
   end

   -- Show Autofill Healer Count
   local hc = 0 
   for k, v in pairs(raidHealers) do
      v.assigned = false
      hc = hc + 1
   end
   local version, build, date, tocversion = GetBuildInfo()
   self:Print("Total Healers in Raid: "..hc)
   
   -- loop through healer name editboxes, if one is blank then add the first non-assigned raid healer
   for i=1, 15 do 
      local name = self.frame.healerNames[i]:GetText() 
      if strlen(name) == 0 then 
         for k, v in pairs(raidHealers) do 
            for j=1, 15 do
               local dupcheck = self.frame.healerNames[j]:GetText()
               if v.name == dupcheck then
               	 v.assigned = true
               end  
             end	
            if v.assigned == false then 
               self:DebugPrint("Adding healer:", v.name, v.assigned)
               self.frame.healerNames[i]:SetText(v.name)
               v.assigned = true 
               break
            end 
         end
      end 
   end
end

function HealComm:IconDragStart(frame)
   frame:StartMoving() 
   frame:SetAlpha(0.75)
end

function HealComm:IconDragStop(frame, healerNumber)
   -- frame is the icon frame that holds the texture 
   -- healerNumber is the row number of the icon that started the drag 
   frame:StopMovingOrSizing() 
   frame:SetAlpha(1)
   frame:ClearAllPoints()
   frame:SetPoint("RIGHT",self.frame.healerNames[healerNumber], "LEFT", -5, 0)
   if frame.texture:GetTexture() == nil then
      return 
   end 
   for k, v in pairs(self.frame.healerIcons) do 
      if MouseIsOver(self.frame.healerIcons[k]) and k ~= healerNumber then
         -- swap healer names  
         local tempName = self.frame.healerNames[k]:GetText()
         self.frame.healerNames[k]:SetText(self.frame.healerNames[healerNumber]:GetText())
         self.frame.healerNames[healerNumber]:SetText(tempName)
         return
      end 
   end 
end

function HealComm:PresetAssignmentDragStop(frame, assignmentIndex)
   frame:StopMovingOrSizing()
   frame:ClearAllPoints()
   frame:SetPoint("TOP", self.frame.presetAssignmentHeadingLabel, "TOP", 0, -25*assignmentIndex)

   -- assign label text to an assignment editbox if the drag stopped over it 
   -- no easy way around this besides looping through all editboxes 
   for phase=1,3 do 
      for row=1,15 do 
         if MouseIsOver(self.frame.assignments["phase"..phase][row]) then 
            -- set value of assignment to the preset label 
            self.frame.assignments["phase"..phase][row]:SetText(frame.text:GetText())
            return 
         end
      end 
   end 
end

function HealComm:UpdatePresetAssignmentTankNames()
   -- hide the labels with tank names in case we have less tanks assigned then previously 
   for i=10,15 do 
      if self.frame.presetAssignments[i] then 
         self.frame.presetAssignments[i].text:SetText("")
         self.frame.presetAssignments[i]:Hide()
      end
   end
   local n = 10 -- counter for tank name labels, starts at 10 as the first 9 are presets 
   for k, v in pairs(raidTanks) do
      -- end at 16 if we get there because the frame only allows for up to 6 tanks 
      if n == 16 then 
         break
      end 
      if not self.frame.presetAssignments[n] then 
         -- create assignment label if it doesn't exist 
         self:CreatePresetAssignment(v, n)
      else 
         -- if the label already exists then we only need to update the text to the tank's name 
         self.frame.presetAssignments[n].text:SetText(v)
         self.frame.presetAssignments[n]:Show()
      end 
      n = n + 1 
   end 
end


---------------------------------------
-- OPTIONS  
---------------------------------------
local options = {
  type = "group",
  args = {
    header1 = {
      name = "Display Options",
      type = "header",
      order = 100,
    },
    minimap = {
      name = "Hide minimap icon                       ",
      desc = "Determines if the minimap icon is displayed.",
      type = "toggle",
      width = "Full",
      set = 
         function(info,val) 
            HealComm.db.profile.minimap.hide = val
            if val == true then 
               LibDBIcon:Hide("HealComm") 
            else
               LibDBIcon:Show("HealComm")
            end 
         end, 
      get = function(info) return HealComm.db.profile.minimap.hide end,
      order = 101,
    },
    highlightMissingHealers = {
      name = "Highlight missing healers",
      desc = "When you are in a raid, healer names that aren't in your raid group will be highlighted in red.  (NOTE: This requires a restart to affect existing names.)",
      type = "toggle",
      width = "Full",
      set = function(info,val) HealComm.db.profile.highlightMissingHealers = val end,
      get = function(info) return HealComm.db.profile.highlightMissingHealers end,
      order = 102,
    },
    header2 = {
      name = "Communications Options",
      type = "header",
      order = 200,
    },
    autoreply = {
      name = "Auto reply assignment",
      desc = "Auto replies with a player's assignment if they whisper you with 'assignment'.",
      type = "toggle",
      width = "Full",
      set = 
         function(info,val) 
            HealComm.db.profile.autoReplyAssignments = val 
            if val == true then 
               HealComm:RegisterEvent("CHAT_MSG_WHISPER", "ReplyWithAssignment")
            else
               HealComm:UnregisterEvent("CHAT_MSG_WHISPER")
            end 
         end,
      get = function(info) return HealComm.db.profile.autoReplyAssignments end,
      order = 201,
    },
    broadcastHealerNamesFirst = {
      name = "Healer Names First",
      desc = "Format broadcasts 'Healer1, Healer2: Assignment' instead of the normal 'Assignment: Healer1, Healer2'.",
      type = "toggle",
      width = "Full",
      set = function(info,val) HealComm.db.profile.broadcastHealerNamesFirst = val end,
      get = function(info) return HealComm.db.profile.broadcastHealerNamesFirst end,
      order = 202,
    },    
  }
}

local AceConfig = LibStub("AceConfig-3.0")
function HealComm:RegisterOptions() 
   AceConfig:RegisterOptionsTable("HealComm", options)
   self.optionFrame = {}
   self.optionFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("HealComm", "HealComm", nil)
end 